{split out from ServerWinshoeSMTP to clean up that unit...}
unit SMTPFUNC;

interface

function GetInternetDateTimeStr(aDateTime : TDateTime): string;
function LogFileDateTime : string;{0.11}
function FixDirBackSlash(const aDir : string): string;
procedure StripToFrom(var AddrStr: string);
function ParseAddress(const FullAddrStr: string;
                      var Domain, PrettyAddress, MachineAddress : string) : boolean;
procedure SetDefaultDateTimeStrings;
function GetFirstTokenDeleteFromArg(var s1: string; const sDelim: string): string;

implementation

uses
  Windows,SysUtils,DEFDATE;

//------------------------------------------------------------------------------
//                        Start Global Code
//------------------------------------------------------------------------------
function GetInternetDateTimeStr(aDateTime : TDateTime): string;
var
  lpTimeZoneInformation: TTimeZoneInformation;
  S,S2 : string;
  B : integer;
  Neg : boolean;
  Return : longint;
begin                           { GetInternetDateTimeStr }
  S := FormatDateTime('ddd, dd mmm yyyy hh":"nn":"ss ',aDateTime);{0.40}
  Return := GetTimeZoneInformation(lpTimeZoneInformation);
  if not (Return = -1) then begin
    B := lpTimeZoneInformation.Bias; {this is the way gettzinfo works, look it up}
    if (Return = TIME_ZONE_ID_DAYLIGHT) then
      B := B + lpTimeZoneInformation.DaylightBias; {0.34}
    B := (-B div 60) * 100;
    Neg := false;
    if (B < 0) then Neg := true;
    S2 := IntToStr(abs(B));
    while (length(S2) < 4) do S2 := '0'+S2;
    if Neg then S2 := '-'+S2;
    S := S + S2;
  end;
  Result := S;
end;                            { GetInternetDateTimeStr }

function LogFileDateTime : string;{0.11}
begin
  Result := '['+FormatDateTime('dd"/"mmm"/"yyyy":"hh":"nn":"ss',Now)+']';{0.40}
end;

function FixDirBackSlash(const aDir : string): string;
begin
  Result := aDir;
  if (length(aDir) > 0) and (AnsiLastChar(aDir)^ <> '\') then
    Result := aDir + '\'
end;

procedure StripToFrom(var AddrStr: string);
var
  UpAddrStr : string;
  IPos : integer;
begin
  UpAddrStr := UpperCase(AddrStr);
  Ipos := Pos('TO:',UpaddrStr);
  if IPos <> 0 then  Delete(AddrStr,Ipos,3)
  else begin
    Ipos := Pos('FROM:',UpAddrStr);
    if Ipos <> 0 then Delete(AddrStr,Ipos,5);
  end;
  AddrStr := Trim(AddrStr);
end;

function CharCount(S : string; C : char) : integer;
var
  Loop : integer;
  Count : integer;
begin
  Count := 0;
  for Loop := 1 to length(S) do begin
    if (S[Loop] = C) then inc(Count);
  end;
  CharCount := Count;
end;

function ParseAddress(const FullAddrStr: string;
                      var Domain, PrettyAddress, MachineAddress : string) : boolean;
{v0.16 major cleanup, now correctly deals with %,: and !}
var
  InAddr : string;
  Comment : string;
  HumanName : string;
  UserName : string;
  P,Q : integer;
  S1,S2 : string;
begin
  {get rid of comments first}
  ParseAddress := true;
  InAddr := FullAddrStr;
  StripToFrom(InAddr);
  P := pos('(',InAddr);
  Q := pos(')',InAddr);
  Comment := '';
  if (P > 0) and (Q > 0) then begin
    Comment := copy(InAddr,1,Q-1);
    delete(Comment,1,P);
    InAddr := copy(InAddr,1,P-1);
  end;
  InAddr := Trim(InAddr);

  {now parse out the machine and human address if both are listed}
  MachineAddress := InAddr;
  P := pos('<',MachineAddress);
  Q := pos('>',MachineAddress);
  if (P > 0) and (Q > 0) then begin {bingo, we got a machine address surrounded by < >}
    MachineAddress := copy(MachineAddress,1,Q-1);
    delete(MachineAddress,1,P);
    InAddr := copy(InAddr,1,P-1) + copy(InAddr,Q+1,length(InAddr));
    InAddr := trim(InAddr);
    HumanName := InAddr;
    while (pos('"',HumanName) > 0) do
      delete(HumanName,pos('"',HumanName),1);
    HumanName := Trim(HumanName);
  end else begin
    MachineAddress := InAddr;
    HumanName := '';
  end;

  {sanitation time}
  if (CharCount(MachineAddress,':') > 1) then begin
    ParseAddress := false;
    exit;
  end;
  P := pos(':',MachineAddress);
  if (P > 0) then {a:b becomes ba}
    MachineAddress := copy(MachineAddress,P+1,255) +
                      copy(MachineAddress,1,P-1);
  while (pos('"',MachineAddress) > 0) do {remove all ", they're useless}
    delete(MachineAddress,pos('"',MachineAddress),1);
  P := pos('%',MachineAddress);
  while (P > 0) do begin {replace all % with @}
    MachineAddress[P] := '@';
    P := pos('%',MachineAddress);
  end;
  P := pos('!',MachineAddress);
  while (P > 0) do begin
  {OK slightly tricky to read this, this procedure does this
   it switches a!b to b@a but only between @ symbols, IE:
   a!b becomes b@a,
   c@a!b@d becomes c@b@a@d
   a!b@d becomes b@a@d
   c@a!b becomes c@b@a}
    S1 := copy(MachineAddress,1,P-1);
    S2 := MachineAddress;
    delete(S2,1,P);
    while (pos('@',S1) > 0) do
      delete(S1,1,pos('@',S1));
    if (pos('@',S2) > 0) then
      S2 := copy(S2,1,pos('@',S2)-1);
    MachineAddress := copy(MachineAddress,1,P-length(S1)-1)+S2+'@'+S1+
                      copy(machineAddress,P+length(S2)+1,255);
    P := pos('!',MachineAddress);
  end;
  {split the username and domain}
  Domain := MachineAddress;
  P := pos('@',Domain);
  if (P > 0) then begin
    delete(Domain,1,P);
    P := pos('@',Domain);
    if (P > 0) then {ignore second @ and everything thereafter, this is a hack to
                     use multipath for relay}
      Domain := copy(Domain,1,P-1);
    P := pos('[',Domain);{strip [] from any domain, should prolly verify
                          valid IP number and replace to maintain standard}
    if (P > 0) then
      delete(Domain,P,1);
    Q := pos(']',Domain);
    if (Q > 0) then
      delete(Domain,Q,1);
    UserName := MachineAddress;
    P := pos('@',UserName);
    UserName := copy(UserName,1,P-1);
    UserName := Trim(UserName);
    if (UserName = '') then UserName := 'trashbin';{default name when none is specified}
  end else begin
    ParseAddress := false;
    UserName := Domain;
    Domain := '';
  end;
  {generate correctly formatted returns}
  MachineAddress := UserName + '@' + Domain;
  PrettyAddress := '';
  if (HumanName <> '') then begin
    PrettyAddress := '"'+HumanName+'" ';
  end;
  PrettyAddress := PrettyAddress + '<' + MachineAddress + '>';
  if (Comment <> '') then begin
    PrettyAddress := PrettyAddress + ' ('+Comment+')';
  end;
end;

procedure SetDefaultDateTimeStrings; {0.40 in defdate}
{fix date/time strings so that they appear american format,
some mailers puke on international formats}
var
  Loop : integer;
begin
  for Loop := 1 to 12 do begin
    ShortMonthNames[Loop] := DefShortMonthNames[Loop];
    LongMonthNames[Loop] := DefLongMonthNames[Loop];
  end;
  for Loop := 1 to 7 do begin
    ShortDayNames[Loop] := DefShortDayNames[Loop];
    LongDayNames[Loop] := DefLongDayNames[Loop]
  end;
end;

function GetFirstTokenDeleteFromArg(var s1: string; const sDelim: string): string;
var
  nPos: integer;
begin                         { GetFirstTokenDeleteFromArg }
  nPos := Pos(sDelim, s1);
  if nPos = 0 then nPos := Length(s1) + 1;
  Result := Copy(s1, 1, nPos - 1);
  Delete(s1, 1, nPos);
  S1 := Trim(S1);
end;                          { GetFirstTokenDeleteFromArg }

end.
